@@ -9,7 +9,7 @@ source "$TOP/tests/common"
 source "$TOP/tests/common.convert"
 
 setup_root_helper
-prepare_test_dev 512M
+prepare_test_dev
 check_prereq btrfs-convert
 check_global_prereq mke2fs
 
